// splash.js for Perlenspiel 3.3
// Composed for the edification of students by Brian Moriarty
// Released under GLPL-3.0

// The following comment lines are for JSHint <https://jshint.com>
/* jshint browser : true, devel : true, esversion : 5, freeze : true */
/* globals PS : true */

// The global variable G is used to encapsulate most game-specific variables and functions
// in an immediately-invoked function expression (IIFE).
// This strategy helps prevent possible clashes with other scripts running in the same page.

var G = ( function () {
	"use strict";

	// CONSTANTS
	// Constant names are all upper-case to make them easy to distinguish
	
	var _GRID_WIDTH = 8;
	var _GRID_HEIGHT = 9;
	var _FRAME_RATE = 20; // seems about right
	var _COLOR_BG = 0x303030; // background color (dark gray)
	var _COLOR_TEXT = 0xFFFFFF; // final status text color (white)
	var _ENGINE_NAME = "P E R L E N S P I E L";

	// The official Perlenspiel logo colors

	var _LOGO_COLOR_ORANGE = 0xF4782B;
	var _LOGO_COLOR_BLUE = 0x6A98BA;
	var _LOGO_COLOR_GREEN = 0xA1C93A;
	var _LOGO_COLOR_YELLOW = 0xEFEB42;
	var _LOGO_COLOR_PURPLE = 0x87578E;
	
	// The "score" played by the game is stored in this array.
	// Each note in the score contains the name of the note to be played,
	// together with the x/y position and color of the associated bead.
	// Note that only some entries in the array contain data! Others are null.
	// The null entries indicate an unused beat in the score.

	var _SCORE = [
		null, // Beat 0

		// Beat 1
		{
			note: "l_hchord_d4",
			x: 0, y: 6,
			color: _LOGO_COLOR_BLUE
		},

		null, // Beat 2

		// Beat 3
		{
			note: "l_hchord_a4",
			x: 1, y: 1,
			color: _LOGO_COLOR_YELLOW
		},

		null, // Beat 4

		// Beat 5
		{
			note: "l_hchord_f4",
			x: 2, y: 3,
			color: _LOGO_COLOR_ORANGE
		},

		null, // Beat 6

		// Beat 7
		{
			note: "l_hchord_d4",
			x: 3, y: 6,
			color: _LOGO_COLOR_BLUE
		},

		null, // Beat 8

		// Beat 9
		{
			note: "l_hchord_db4",
			x: 4, y: 7,
			color: _LOGO_COLOR_PURPLE
		},

		null, // Beat 10

		// Beat 11
		{
			note: "hchord_d4",
			x: 5, y: 6,
			color: _LOGO_COLOR_BLUE
		},

		// Beat 12
		{
			note: "hchord_e4",
			x: 6, y: 4,
			color: _LOGO_COLOR_GREEN
		},

		// Beat 13
		{
			note: "l_hchord_f4",
			x: 7, y: 3,
			color: _LOGO_COLOR_ORANGE
		}
	];

	var _SCORE_LEN = _SCORE.length; // save length of score array for later reference

	// VARIABLES
	// Variable names are expressed in lower-case with camelCaps

	var _noteCnt = 0; // stores total number of active notes in score
	var _loadCnt = 0; // keeps track of how many note files have been loaded
	var _timerID = ""; // timer ID, saved so that timer can be stopped when no longer needed
	var _tick = 0; // the metronome beat counter, used to keep track of where we are in the score
	var _done = false; // set to true when melody playback is complete

	// FUNCTIONS
	// Function names are expressed in lower case with camelCaps

	// Show engine text

	var showText = function () {
		var i, beat;

		PS.statusText( _ENGINE_NAME );
		PS.statusFade( 120 ); // set up 2-second fade-up
		PS.statusColor( _COLOR_TEXT ); // color assignment starts the fade

		// Set up beads for white flash when touched.
		// Ignore empty score entries.

		for ( i = 0; i < _SCORE_LEN; i += 1 ) {
			beat = _SCORE[ i ];
			if ( beat ) {
				PS.fade( beat.x, beat.y, 60, { rgb : PS.COLOR_WHITE } ); // set 1 sec white flash
			}
		}
	};

	// Timer function
	// Not started until all audio files are loaded
	// Called once every G._FRAME_RATE ticks (= 20, 1/3 second)
	// Sequentially plays the notes and draws the beads defined in the _SCORE array

	var metronome = function() {
		var beat;

		// Get the next beat in score array

		beat = _SCORE[ _tick ];

		// Only play non-empty array entries

		if ( beat ) {
			PS.audioPlay( beat.note ); // play the note
			PS.color( beat.x, beat.y, beat.color ); // change the bead color
		}

		_tick += 1; // point to next beat

		// Played the last note?
		// If so, show Perlenspiel banner

		if ( _tick >= _SCORE_LEN ) {
			PS.timerStop( _timerID ); // stop the metronome
			showText(); // show the text, setup bead flashing
			_done = true; // finished! this flag allows beads to be touched and played
		}
	};

	// Audio file loading function (shared by all audio files).
	// This gets called each time a note in the _SCORE object is done loading.
	// It starts the metronome when all notes are loaded and ready to play.

	var _loader = function () {
		_loadCnt += 1;
		if ( _loadCnt >= _noteCnt ) {
			_timerID = PS.timerStart( _FRAME_RATE, metronome );
		}
	};

	// The _PUBLIC object exposes any required variables/methods to Perlenspiel's event handlers.

	var _PUBLIC = {
		init : function ( system ) {
			var i, beat;

			PS.gridSize( _GRID_WIDTH, _GRID_HEIGHT ); // set up 8 x 9 grid
			PS.border( PS.ALL, PS.ALL, 0 ); // hide all bead borders
			PS.gridColor( _COLOR_BG ); // set background color to dark gray
			PS.color( PS.ALL, PS.ALL, _COLOR_BG ); // set all beads to background color
			PS.statusColor( _COLOR_BG ); // set status text color to background color, too

			// If running on a mobile browser, more than one sound file cannot be played at a time.
			// So skip the melody and just show the logo.

			if ( system.host.mobile ) {
				for ( i = 0; i < _SCORE_LEN; i += 1 ) {
					beat = _SCORE[ i ];
					if ( beat )	{
						PS.color( beat.x, beat.y, beat.color );
					}
				}
				showText();
				return;
			}

			// Scan the score, load all of the notes, and save each non-null beat in the associated bead's data.
			// This makes it easy to play the correct note when a bead is touched.

			_noteCnt = 0; // init note counter

			for ( i = 0; i < _SCORE_LEN; i += 1 ) {
				beat = _SCORE[ i ];

				// Ignore empty score entries

				if ( beat ) {
					// Save the beat object in bead's private data for later reference

					PS.data( beat.x, beat.y, beat );

					// Establish a 1-second fade-in on this bead

					PS.fade( beat.x, beat.y, 60 );
					
					_noteCnt += 1; // increment the note total
					
					// Load the note. PS.audioLoad() is smart enough not to reload a sound file
					// it has already loaded, so no need to worry about duplicate notes.
					// Remember: There is no guarantee that notes will actually be loaded
					// in the same order as requested! All asynchronous functions must accommodate
					// the non-deterministic order of asset loading.
					
					PS.audioLoad( beat.note, {
						lock : true,
						onLoad : _loader
					} );
				}
			}
		},

		touch : function ( x, y, data ) {
			// Only note beads will have an object assigned to their private data.
			// This assignment happens in G.init() above.
			// All other bead data will contain their default value (zero, 0).

			if ( data && ( typeof data === "object" ) ) {

				// Only respond to touches if melody is done playing

				if ( _done ) {
					PS.color( x, y, data.color ); // flash current color using white fade
					PS.audioPlay( data.note ); // play note
				}
			}
		}
	};

	// Assign the _PUBLIC object to the G global by returning it as the value of this IIFE.
	// ONLY the two properties of _PUBLIC (.init and .touch) will be exposed to the global namespace
	// as properties of G. All other variables/functions defined within the IIFE are invisibly encapsulated.

	return _PUBLIC;
} () );

// Tell PS.init() to call G.init() when the engine is initialized and ready to run.
// Note that NO parentheses are used to assign G.init() to PS.init()!
// Doing so would cause G.init() to be called as soon as this JavaScript file is loaded.

PS.init = G.init;

// Tell PS.touch() to call G.touch() when a bead is touched.
// Again, no parens on G.touch() reference to avoid immediate execution on file load.

PS.touch = G.touch;

// The other Perlenspiel event functions aren't used in this app.
// Their templates can be safely deleted to save space and keep things tidy.

